// BSTree.h ... implementation of binary search tree ADT
// Written by John Shepherd, March 2013

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "BSTree.h"

typedef struct BSTNode *Link;

typedef struct BSTNode {
	int  value;
	Link left, right;
} BSTNode;

// make a new node containing a value
static
Link newBSTNode(int v)
{
	Link new = malloc(sizeof(BSTNode));
	assert(new != NULL);
	new->value = v;
	new->left = new->right = NULL;
	return new;
}

// create a new empty BSTree
BSTree newBSTree()
{
	return NULL;
}

// free memory associated with BSTree
void dropBSTree(BSTree t)
{
	if (t == NULL)
		/* do nothing */;
	else {
		dropBSTree(t->left);
		dropBSTree(t->right);
		free(t);
	}
}

#if 0
// display a BSTree (sideways)
static
void doShowBSTree(BSTree t, int level)
{
	int i;
	if (t == NULL) return;
	doShowBSTree(t->right, level+1);
	for (i = 0; i < level; i++) printf("   ");
	printf("%d\n", t->value);
	doShowBSTree(t->left, level+1);
}
#endif
void showBSTree(BSTree t)
{
	void doShowTree(BSTree);
	doShowTree(t);
#if 0
    //doShowBSTree(t,0);
#endif
}

// compute depth of BSTree
int BSTreeDepth(BSTree t)
{
	if (t == NULL)
		return 0;
	else {
		int ldepth = BSTreeDepth(t->left);
		int rdepth = BSTreeDepth(t->right);
		//return 1 + (ldepth > rdepth)?ldepth:rdepth;
		if (ldepth > rdepth)
			return 1+ldepth;
		else
			return 1+rdepth;
	}
}

// count #nodes in BSTree
int BSTreeNumNodes(BSTree t)
{
	if (t == NULL)
		return 0;
	else
		return 1 + BSTreeNumNodes(t->left)
		         + BSTreeNumNodes(t->right);
}

// check whether a value is in a BSTree
int BSTreeFind(BSTree t, int v)
{
	if (t == NULL)
		return 0;
	else if (v < t->value)
		return BSTreeFind(t->left, v);
	else if (v > t->value)
		return BSTreeFind(t->right, v);
	else // (v == t->value)
		return 1;
}

int BSTreeFindI(BSTree t, int v)
{
	Link cur;
	cur = t;
	while (cur != NULL) {
		if (v < cur->value)
			cur = cur->left;
		else if (v > cur->value)
			cur = cur->right;
		else // (v == cur->value)
			return 1;
	}
	return 0;
}

// insert a new value into a BSTree
BSTree BSTreeInsert(BSTree t, int v)
{
	if (t == NULL)
		t = newBSTNode(v);
	else if (v < t->value)
		t->left = BSTreeInsert(t->left, v);
	else if (v > t->value)
		t->right = BSTreeInsert(t->right, v);
	return t;
}

BSTree BSTreeInsertI(BSTree t, int v)
{
	Link cur, parent;
	if (t == NULL) return newBSTNode(v);
	cur = t; parent = NULL;
	while (cur != NULL) {
		parent = cur;
		if (v < cur->value)
			cur = cur->left;
		else if (v > cur->value)
			cur = cur->right;
		else
			return t;
	}
	Link new = newBSTNode(v);
	if (v < parent->value)
		parent->left = new;
	else
		parent->right = new;
	return t;
}


// generic traversal
void BSTreeTraverse(BSTree t,
				    Visit visit,
                    //void (*visit)(struct BSTNode *),
                    char *style)
{
	if (t == NULL) return;
	if (strcmp(style,"NLR") == 0) (*visit)(t);
	BSTreeTraverse(t->left, visit, style);
	if (strcmp(style,"LNR") == 0) (*visit)(t);
	BSTreeTraverse(t->right, visit, style);
	if (strcmp(style,"LRN") == 0) (*visit)(t);
}



// delete a value from a BSTree
BSTree BSTreeDelete(BSTree t, int v)
{
	BSTree deleteRoot(BSTree);

	if (t == NULL) {
		printf("delete from an empty tree ... easy\n");
		return NULL;
	}
	else if (v < t->value) {
		printf("delete %d from tree with root %d\n",v,t->value);
		t->left = BSTreeDelete(t->left, v);
	}
	else if (v > t->value) {
		printf("delete %d from tree with root %d\n",v,t->value);
		t->right = BSTreeDelete(t->right, v);
	}
	else { // (v == t->value)
		printf("delete root %d and rearrange\n",v);
		t = deleteRoot(t);
	}
	return t;
}

// delete root of tree
BSTree deleteRoot(BSTree t)
{
	Link newRoot;
	// if no subtrees, tree empty after delete
	if (t->left == NULL && t->right == NULL) {
		free(t);
		return NULL;
	}
	// if only right subtree, make it the new root
	else if (t->left == NULL && t->right != NULL) {
		newRoot = t->right;
		free(t);
		return newRoot;
	}
	// if only left subtree, make it the new root
	else if (t->left != NULL && t->right == NULL) {
		newRoot = t->left;
		free(t);
		return newRoot;
	}
	// else (t->left != NULL && t->right != NULL)
	// so has two subtrees
	// - find inorder successor (grab value)
	// - delete inorder successor node
	// - move its value to root
	Link parent = t;
	Link succ = t->right; // not null!
	while (succ->left != NULL) {
		parent = succ;
		succ = succ->left;
	}
	int succVal = succ->value;
	printf("find successor %d\n",succVal);
	t = BSTreeDelete(t,succVal);
	t->value = succVal;
	return t;
}

// Pretty tree printer

#include "show.c"
